==> heptapod-rails-0.16.1.tar.gz: heptapod-rails-0.16.1/app/controllers/import/bitbucket_controller.rb <==
# frozen_string_literal: true

class Import::BitbucketController < Import::BaseController
  extend ::Gitlab::Utils::Override

  include ActionView::Helpers::SanitizeHelper

  before_action :verify_bitbucket_import_enabled
  before_action :bitbucket_auth, except: :callback

  rescue_from OAuth2::Error, with: :bitbucket_unauthorized
  rescue_from Bitbucket::Error::Unauthorized, with: :bitbucket_unauthorized

  def callback
    response = oauth_client.auth_code.get_token(params[:code], redirect_uri: users_import_bitbucket_callback_url)

    session[:bitbucket_token]         = response.token
    session[:bitbucket_expires_at]    = response.expires_at
    session[:bitbucket_expires_in]    = response.expires_in
    session[:bitbucket_refresh_token] = response.refresh_token

    redirect_to status_import_bitbucket_url
  end

  def status
    super
  end

  def realtime_changes
    super
  end

  def import_form
    target_namespace = Namespace.find(params[:namespace_id].to_i)
    begin
      project = create_from_bb_rpath(params[:repo_id].to_s, target_namespace)
    rescue StandardError => err
      redirect_to(status_import_bitbucket_path, status: 303, alert: "Error: #{err}")
      return
    end

    if project.nil?
      redirect_to(status_import_bitbucket_path,
                  status: 303,
                  alert: _('This namespace has already been taken! Please choose another one.'),
                 )
    elsif project.persisted?
      redirect_to(status_import_bitbucket_path, status: 303, notice: _('Project import started'))
    else
      err = project_save_error(project)
      Rails.logger.info("testgr Bitbucket controller project err=#{err}")
      redirect_to(status_import_bitbucket_path, status: 303, alert: "Error: #{err}")
    end
  end

  def create
    repo_id = params[:repo_id].to_s
    project = create_from_bb_rpath(repo_id.gsub('___', '/'))
    if project.nil?
      render json: { errors: _('This namespace has already been taken! Please choose another one.') }, status: :unprocessable_entity
    else
      if project.persisted?
        render json: ProjectSerializer.new.represent(project, serializer: :import)
      else
        render json: { errors: project_save_error(project) }, status: :unprocessable_entity
      end
    end
  end

  # name is the rpath in Bitbucket Cloud
  def create_from_bb_rpath(name, target_namespace = nil)
    bitbucket_client = Bitbucket::Client.new(credentials)

    repo = bitbucket_client.repo(name)
    project_name = params[:new_name].presence || repo.name
    if target_namespace.nil?
      repo_owner = repo.owner
      repo_owner = current_user.username if repo_owner == bitbucket_client.user.username
      namespace_path = params[:new_namespace].presence || repo_owner
      target_namespace = find_or_create_namespace(namespace_path, current_user)
    end

    # The token in a session can be expired, we need to get most recent one because
    # Bitbucket::Connection class refreshes it.
    session[:bitbucket_token] = bitbucket_client.connection.token
    project = Project.where(namespace_id: target_namespace.id,
                            path: project_name).first
    if !project.nil?
      record_url = URI(project.import_url)
      current_url = URI(repo.clone_url)
      if record_url.host != current_url.host || record_url.path != current_url.path
        raise StandardError,
              "Found project '#{project_name}' in target namespace "\
              "'#{target_namespace.full_path}', but its previous import URL "\
              "#{record_url} doesn't match the expected #{current_url}"
      end
      project.create_or_update_import_data(credentials: credentials)
      project.save
      Rails.logger.info("Bitbucket controller: scheduling asynchronous "\
                        "reimport for project #{project.id}")

      BitbucketReimportWorker.perform_async(project.id)
      project
    elsif current_user.can?(:create_projects, target_namespace)

      Gitlab::BitbucketImport::ProjectCreator.new(repo, project_name, target_namespace, current_user, credentials).execute
    end
  end

  protected

  # rubocop: disable CodeReuse/ActiveRecord
  override :importable_repos
  def importable_repos
    already_added_projects_names = already_added_projects.map(&:import_source)

    bitbucket_repos.reject { |repo| already_added_projects_names.include?(repo.full_name) || !repo.valid? }
  end
  # rubocop: enable CodeReuse/ActiveRecord

  override :incompatible_repos
  def incompatible_repos
    bitbucket_repos.reject { |repo| repo.valid? }
  end

  override :provider_name
  def provider_name
    :bitbucket
  end

  override :provider_url
  def provider_url
    provider.url
  end

  private

  def oauth_client
    @oauth_client ||= OAuth2::Client.new(provider.app_id, provider.app_secret, options)
  end

  def provider
    Gitlab::Auth::OAuth::Provider.config_for('bitbucket_import',
                                             fallback='bitbucket')
  end

  def client
    @client ||= Bitbucket::Client.new(credentials)
  end

  def bitbucket_repos
    @bitbucket_repos ||= client.repos(filter: sanitized_filter_param).to_a
  end

  def options
    OmniAuth::Strategies::Bitbucket.default_options[:client_options].deep_symbolize_keys
  end

  def verify_bitbucket_import_enabled
    render_404 unless bitbucket_import_enabled?
  end

  def bitbucket_auth
    go_to_bitbucket_for_permissions if session[:bitbucket_token].blank?
  end

  def go_to_bitbucket_for_permissions
    redirect_to oauth_client.auth_code.authorize_url(redirect_uri: users_import_bitbucket_callback_url)
  end

  def bitbucket_unauthorized
    go_to_bitbucket_for_permissions
  end

  def credentials
    {
      token: session[:bitbucket_token],
      expires_at: session[:bitbucket_expires_at],
      expires_in: session[:bitbucket_expires_in],
      refresh_token: session[:bitbucket_refresh_token]
    }
  end

  def sanitized_filter_param
    @filter ||= sanitize(params[:filter])
  end
end

==> heptapod-rails-0.25.2.tar.gz: heptapod-rails-0.25.2/app/controllers/import/bitbucket_controller.rb <==
# frozen_string_literal: true

class Import::BitbucketController < Import::BaseController
  extend ::Gitlab::Utils::Override

  include ActionView::Helpers::SanitizeHelper

  before_action :verify_bitbucket_import_enabled
  before_action :bitbucket_auth, except: :callback

  rescue_from OAuth2::Error, with: :bitbucket_unauthorized
  rescue_from Bitbucket::Error::Unauthorized, with: :bitbucket_unauthorized

  def callback
    response = oauth_client.auth_code.get_token(params[:code], redirect_uri: users_import_bitbucket_callback_url)

    session[:bitbucket_token]         = response.token
    session[:bitbucket_expires_at]    = response.expires_at
    session[:bitbucket_expires_in]    = response.expires_in
    session[:bitbucket_refresh_token] = response.refresh_token

    redirect_to status_import_bitbucket_url
  end

  def status
    super
  end

  def realtime_changes
    super
  end

  def create
    bitbucket_client = Bitbucket::Client.new(credentials)

    repo_id = params[:repo_id].to_s
    name = repo_id.gsub('___', '/')
    repo = bitbucket_client.repo(name)
    project_name = params[:new_name].presence || repo.name

    repo_owner = repo.owner
    repo_owner = current_user.username if repo_owner == bitbucket_client.user.username
    namespace_path = params[:new_namespace].presence || repo_owner
    target_namespace = find_or_create_namespace(namespace_path, current_user)

    if current_user.can?(:create_projects, target_namespace)
      # The token in a session can be expired, we need to get most recent one because
      # Bitbucket::Connection class refreshes it.
      session[:bitbucket_token] = bitbucket_client.connection.token

      project = Gitlab::BitbucketImport::ProjectCreator.new(repo, project_name, target_namespace, current_user, credentials).execute

      if project.persisted?
        render json: ProjectSerializer.new.represent(project, serializer: :import)
      else
        render json: { errors: project_save_error(project) }, status: :unprocessable_entity
      end
    else
      render json: { errors: _('This namespace has already been taken! Please choose another one.') }, status: :unprocessable_entity
    end
  end

  protected

  # rubocop: disable CodeReuse/ActiveRecord
  override :importable_repos
  def importable_repos
    already_added_projects_names = already_added_projects.map(&:import_source)

    bitbucket_repos.reject { |repo| already_added_projects_names.include?(repo.full_name) || !repo.valid? }
  end
  # rubocop: enable CodeReuse/ActiveRecord

  override :incompatible_repos
  def incompatible_repos
    bitbucket_repos.reject { |repo| repo.valid? }
  end

  override :provider_name
  def provider_name
    :bitbucket
  end

  override :provider_url
  def provider_url
    provider.url
  end

  private

  def oauth_client
    @oauth_client ||= OAuth2::Client.new(provider.app_id, provider.app_secret, options)
  end

  def provider
    Gitlab::Auth::OAuth::Provider.config_for('bitbucket')
  end

  def client
    @client ||= Bitbucket::Client.new(credentials)
  end

  def bitbucket_repos
    @bitbucket_repos ||= client.repos(filter: sanitized_filter_param).to_a
  end

  def options
    OmniAuth::Strategies::Bitbucket.default_options[:client_options].deep_symbolize_keys
  end

  def verify_bitbucket_import_enabled
    render_404 unless bitbucket_import_enabled?
  end

  def bitbucket_auth
    go_to_bitbucket_for_permissions if session[:bitbucket_token].blank?
  end

  def go_to_bitbucket_for_permissions
    redirect_to oauth_client.auth_code.authorize_url(redirect_uri: users_import_bitbucket_callback_url)
  end

  def bitbucket_unauthorized
    go_to_bitbucket_for_permissions
  end

  def credentials
    {
      token: session[:bitbucket_token],
      expires_at: session[:bitbucket_expires_at],
      expires_in: session[:bitbucket_expires_in],
      refresh_token: session[:bitbucket_refresh_token]
    }
  end
end
